@@ -100,8 +100,8 @@ void __igt_fixture_end(void) __attribute__((noreturn));
  * Annotate global test fixture code
  *
  * Testcase with subtests often need to set up a bunch of global state as the
- * common test fixture. To avoid such code interferring with the subtest
- * enumeration (e.g. when enumerating on systemes without an intel gpu) such
+ * common test fixture. To avoid such code interfering with the subtest
+ * enumeration (e.g. when enumerating on systems without an intel gpu) such
  * blocks should be annotated with igt_fixture.
  */
 #define igt_fixture for (int igt_tokencat(__tmpint,__LINE__) = 0; \
@@ -131,7 +131,7 @@ int igt_subtest_init_parse_opts(int *argc, char **argv,
  * @argv: argv from the test's main()
  *
  * This initializes the for tests with subtests without the need for additional
- * cmdline options. It is just a simplified version of
+ * command line options. It is just a simplified version of
  * igt_subtest_init_parse_opts().
  *
  * If there's not a reason to the contrary it's less error prone to just use an
@@ -199,7 +199,7 @@ bool igt_only_list_subtests(void);
  *
  * This is a magic control flow block used instead of a main() function for
  * tests with subtests. Open-coding the main() function is only recommended if
- * the test needs to parse additional cmdline arguments of its own.
+ * the test needs to parse additional command line arguments of its own.
  */
 #define igt_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
@@ -239,7 +239,7 @@ void igt_simple_init_parse_opts(int *argc, char **argv,
  *
  * This is a magic control flow block used instead of a main() function for
  * simple tests. Open-coding the main() function is only recommended if
- * the test needs to parse additional cmdline arguments of its own.
+ * the test needs to parse additional command line arguments of its own.
  */
 #define igt_simple_main \
 	static void igt_tokencat(__real_main, __LINE__)(void); \
@@ -346,6 +346,15 @@ void igt_exit(void) __attribute__((noreturn));
 				  "error: %d " #ncmp " %d\n", __n1, __n2); \
 	} while (0)
 
+/**
+ * igt_assert_cmpuint:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpint(), but for unsigned ints.
+ */
 #define igt_assert_cmpuint(n1, cmp, ncmp, n2) \
 	do { \
 		uint32_t __n1 = (n1), __n2 = (n2); \
@@ -355,6 +364,42 @@ void igt_exit(void) __attribute__((noreturn));
 				  "error: %#x " #ncmp " %#x\n", __n1, __n2); \
 	} while (0)
 
+/**
+ * igt_assert_cmpu64:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpuint(), but for larger ints.
+ */
+#define igt_assert_cmpu64(n1, cmp, ncmp, n2) \
+	do { \
+		uint64_t __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %#llx " #ncmp " %#llx\n", (long long)__n1, (long long)__n2); \
+	} while (0)
+
+/**
+ * igt_assert_cmpdouble:
+ * @n1: first value
+ * @cmp: compare operator
+ * @ncmp: negated version of @cmp
+ * @n2: second value
+ *
+ * Like igt_assert_cmpint(), but for doubles.
+ */
+#define igt_assert_cmpdouble(n1, cmp, ncmp, n2) \
+	do { \
+		double __n1 = (n1), __n2 = (n2); \
+		if (__n1 cmp __n2) ; else \
+		__igt_fail_assert(IGT_LOG_DOMAIN, __FILE__, __LINE__, __func__, \
+				  #n1 " " #cmp " " #n2, \
+				  "error: %#lf " #ncmp " %#lf\n", __n1, __n2); \
+	} while (0)
+
 /**
  * igt_assert_eq:
  * @n1: first integer
@@ -367,8 +412,34 @@ void igt_exit(void) __attribute__((noreturn));
  * of simply printing the stringified expression.
  */
 #define igt_assert_eq(n1, n2) igt_assert_cmpint(n1, ==, !=, n2)
+
+/**
+ * igt_assert_eq_u32:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_eq(), but for uint32_t.
+ */
 #define igt_assert_eq_u32(n1, n2) igt_assert_cmpuint(n1, ==, !=, n2)
 
+/**
+ * igt_assert_eq_u64:
+ * @n1: first integer
+ * @n2: second integer
+ *
+ * Like igt_assert_eq_u32(), but for uint64_t.
+ */
+#define igt_assert_eq_u64(n1, n2) igt_assert_cmpu64(n1, ==, !=, n2)
+
+/**
+ * igt_assert_eq_double:
+ * @n1: first double
+ * @n2: second double
+ *
+ * Like igt_assert_eq(), but for doubles.
+ */
+#define igt_assert_eq_double(n1, n2) igt_assert_cmpdouble(n1, ==, !=, n2)
+
 /**
  * igt_assert_neq:
  * @n1: first integer
@@ -420,7 +491,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_require(expr) do { \
 	if (!(expr)) igt_skip_check(#expr , NULL); \
-	else igt_debug("Test requirement passed: "#expr"\n"); \
+	else igt_debug("Test requirement passed: %s\n", #expr); \
 } while (0)
 
 /**
@@ -435,7 +506,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_skip_on(expr) do { \
 	if ((expr)) igt_skip_check("!(" #expr ")" , NULL); \
-	else igt_debug("Test requirement passed: !("#expr")\n"); \
+	else igt_debug("Test requirement passed: !(%s)\n", #expr); \
 } while (0)
 
 /**
@@ -454,7 +525,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_require_f(expr, f...) do { \
 	if (!(expr)) igt_skip_check(#expr , f); \
-	else igt_debug("Test requirement passed: "#expr"\n"); \
+	else igt_debug("Test requirement passed: %s\n", #expr); \
 } while (0)
 
 /**
@@ -473,7 +544,7 @@ void igt_exit(void) __attribute__((noreturn));
  */
 #define igt_skip_on_f(expr, f...) do { \
 	if ((expr)) igt_skip_check("!("#expr")", f); \
-	else igt_debug("Test requirement passed: !("#expr")\n"); \
+	else igt_debug("Test requirement passed: !(%s)\n", #expr); \
 } while (0)
 
 /* fork support code */
@@ -661,7 +732,10 @@ extern enum igt_log_level igt_log_level;
 	} while (0)
 
 
-void igt_set_timeout(unsigned int seconds);
+void igt_set_timeout(unsigned int seconds,
+		     const char *op);
+
+void igt_reset_timeout(void);
 
 FILE *__igt_fopen_data(const char* igt_srcdir, const char* igt_datadir,
 		       const char* filename);
