@@ -182,6 +182,7 @@ typedef struct {
 	unsigned int position_changed : 1;
 	unsigned int panning_changed  : 1;
 	unsigned int rotation_changed : 1;
+	unsigned int size_changed     : 1;
 	/*
 	 * drm_plane can be NULL for primary and cursor planes (when not
 	 * using the atomic modeset API)
@@ -207,6 +208,9 @@ struct igt_pipe {
 #define IGT_MAX_PLANES	4
 	int n_planes;
 	igt_plane_t planes[IGT_MAX_PLANES];
+	uint64_t background; /* Background color MSB BGR 16bpc LSB */
+	uint32_t background_changed : 1;
+	uint32_t background_property;
 };
 
 typedef struct {
@@ -217,6 +221,8 @@ typedef struct {
 	char *name;
 	bool valid;
 	unsigned long pending_crtc_idx_mask;
+	bool use_override_mode;
+	drmModeModeInfo override_mode;
 } igt_output_t;
 
 struct igt_display {
@@ -239,6 +245,7 @@ int  igt_display_get_n_pipes(igt_display_t *display);
 
 const char *igt_output_name(igt_output_t *output);
 drmModeModeInfo *igt_output_get_mode(igt_output_t *output);
+void igt_output_override_mode(igt_output_t *output, drmModeModeInfo *mode);
 void igt_output_set_pipe(igt_output_t *output, enum pipe pipe);
 igt_plane_t *igt_output_get_plane(igt_output_t *output, enum igt_plane plane);
 
@@ -252,6 +259,11 @@ void igt_plane_set_position(igt_plane_t *plane, int x, int y);
 void igt_plane_set_size(igt_plane_t *plane, int w, int h);
 void igt_plane_set_panning(igt_plane_t *plane, int x, int y);
 void igt_plane_set_rotation(igt_plane_t *plane, igt_rotation_t rotation);
+void igt_crtc_set_background(igt_pipe_t *pipe, uint64_t background);
+void igt_fb_set_position(struct igt_fb *fb, igt_plane_t *plane,
+	uint32_t x, uint32_t y);
+void igt_fb_set_size(struct igt_fb *fb, igt_plane_t *plane,
+	uint32_t w, uint32_t h);
 
 void igt_wait_for_vblank(int drm_fd, enum pipe pipe);
 
@@ -262,6 +274,10 @@ void igt_wait_for_vblank(int drm_fd, enum pipe pipe);
 #define for_each_pipe(display, pipe)					\
 	for (pipe = 0; pipe < igt_display_get_n_pipes(display); pipe++)	\
 
+#define for_each_plane_on_pipe(display, pipe, plane)			\
+	for (int i__ = 0; (plane) = &(display)->pipes[(pipe)].planes[i__], \
+		     i__ < (display)->pipes[(pipe)].n_planes; i__++)
+
 /*
  * Can be used with igt_output_set_pipe() to mean we don't care about the pipe
  * that should drive this output
