@@ -35,6 +35,7 @@
  * Based on gem_gtt_concurrent_blt.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -47,15 +48,7 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "igt_aux.h"
-#include "igt_core.h"
-#include "igt_gt.h"
 #include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
 
 IGT_TEST_DESCRIPTION("Test of pread/pwrite/mmap behavior when writing to active"
 		     " buffers.");
@@ -830,7 +823,7 @@ static void run_forked(struct buffers *buffers,
 	igt_fork(child, 16) {
 		/* recreate process local variables */
 		buffers->count = 0;
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 
 		batch = buffers_init(buffers, buffers->mode, fd);
 
@@ -1079,7 +1072,7 @@ igt_main
 		all = true;
 
 	igt_fixture {
-		fd = drm_open_any();
+		fd = drm_open_driver(DRIVER_INTEL);
 		devid = intel_get_drm_devid(fd);
 		gen = intel_gen(devid);
 		rendercopy = igt_get_render_copyfunc(devid);
