@@ -25,6 +25,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <sys/ioctl.h>
 #include <stdlib.h>
 #include <string.h>
@@ -33,10 +34,7 @@
 #include <pthread.h>
 #include <errno.h>
 
-#include "drmtest.h"
-#include "ioctl_wrappers.h"
 #include "intel_bufmgr.h"
-#include "igt_debugfs.h"
 
 IGT_TEST_DESCRIPTION("Check for flink/open vs. gem close races.");
 
@@ -103,7 +101,7 @@ static void test_flink_name(void)
 
 	threads = calloc(num_threads, sizeof(pthread_t));
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	for (i = 0; i < num_threads; i++) {
 		r = pthread_create(&threads[i], NULL,
@@ -164,7 +162,7 @@ static void test_flink_close(void)
 
 	/* Allocate exit handler fds in here so that we dont screw
 	 * up the counts */
-	fake = drm_open_any();
+	fake = drm_open_driver(DRIVER_INTEL);
 
 	gem_quiescent_gpu(fake);
 	obj_count = get_object_count();
@@ -173,7 +171,7 @@ static void test_flink_close(void)
 
 	threads = calloc(num_threads, sizeof(pthread_t));
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	for (i = 0; i < num_threads; i++) {
 		r = pthread_create(&threads[i], NULL,
