@@ -30,6 +30,7 @@
  * This test checks whether gtt tlbs for cpu access are correctly invalidated.
  */
 
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -40,9 +41,9 @@
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include "drm.h"
-#include "i915_drm.h"
-#include "drmtest.h"
-#include "intel_gpu_tools.h"
+
+IGT_TEST_DESCRIPTION("Check whether gtt tlbs for cpu access are correctly"
+		     " invalidated.");
 
 #define OBJ_SIZE (1024*1024)
 
@@ -76,7 +77,7 @@ igt_simple_main
 
 	igt_skip_on_simulation();
 
-	fd = drm_open_any();
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	handle = gem_create(fd, OBJ_SIZE);
 
