@@ -26,6 +26,7 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -39,12 +40,8 @@
 
 #include <drm.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
-#include "intel_chipset.h"
-#include "intel_io.h"
-#include "igt_debugfs.h"
-#include "igt_aux.h"
+
+IGT_TEST_DESCRIPTION("Test kernel relocations vs. gpu races.");
 
 /*
  * Testcase: Kernel relocations vs. gpu races
@@ -97,7 +94,7 @@ static void create_special_bo(void)
 		BATCH(CMD_POLY_STIPPLE_OFFSET << 16);
 		BATCH(0);
 	}
-	igt_assert(len % 2 == 0);
+	igt_assert_eq(len % 2, 0);
 	BATCH(MI_NOOP);
 	BATCH(MI_BATCH_BUFFER_END);
 
@@ -117,22 +114,20 @@ static void emit_dummy_load(int pitch)
 	}
 
 	for (i = 0; i < 10; i++) {
-		BLIT_COPY_BATCH_START(devid, tile_flags);
+		BLIT_COPY_BATCH_START(tile_flags);
 		OUT_BATCH((3 << 24) | /* 32 bits */
 			  (0xcc << 16) | /* copy ROP */
 			  pitch);
 		OUT_BATCH(0 << 16 | 1024);
 		OUT_BATCH((2048) << 16 | (2048));
 		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-		BLIT_RELOC_UDW(devid);
 		OUT_BATCH(0 << 16 | 0);
 		OUT_BATCH(pitch);
 		OUT_RELOC_FENCED(dummy_bo, I915_GEM_DOMAIN_RENDER, 0, 0);
-		BLIT_RELOC_UDW(devid);
 		ADVANCE_BATCH();
 
-		if (intel_gen(devid) >= 6) {
-			BEGIN_BATCH(3);
+		if (batch->gen >= 6) {
+			BEGIN_BATCH(3, 0);
 			OUT_BATCH(XY_SETUP_CLIP_BLT_CMD);
 			OUT_BATCH(0);
 			OUT_BATCH(0);
@@ -142,35 +137,28 @@ static void emit_dummy_load(int pitch)
 	intel_batchbuffer_flush(batch);
 }
 
-static void faulting_reloc_and_emit(int fd, drm_intel_bo *target_bo)
+static void reloc_and_emit(int fd, drm_intel_bo *target_bo, bool faulting_reloc)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec[2];
 	struct drm_i915_gem_relocation_entry reloc[1];
 	uint32_t handle_relocs;
 	void *gtt_relocs;
-	int ring;
 
-	if (intel_gen(devid) >= 6)
-		ring = I915_EXEC_BLT;
-	else
-		ring = 0;
+	memset(&execbuf, 0, sizeof(execbuf));
+	memset(exec, 0, sizeof(exec));
+	memset(reloc, 0, sizeof(reloc));
 
 	exec[0].handle = target_bo->handle;
-	exec[0].relocation_count = 0;
-	exec[0].relocs_ptr = 0;
-	exec[0].alignment = 0;
-	exec[0].offset = 0;
-	exec[0].flags = 0;
-	exec[0].rsvd1 = 0;
-	exec[0].rsvd2 = 0;
 
 	reloc[0].offset = special_reloc_ofs;
-	reloc[0].delta = 0;
 	reloc[0].target_handle = target_bo->handle;
 	reloc[0].read_domains = I915_GEM_DOMAIN_RENDER;
 	reloc[0].write_domain = I915_GEM_DOMAIN_RENDER;
-	reloc[0].presumed_offset = 0;
+	/* We do not track the last patched value, so force the relocation
+	 * every time.
+	 */
+	reloc[0].presumed_offset = -1;
 
 	handle_relocs = gem_create(fd, 4096);
 	gem_write(fd, handle_relocs, 0, reloc, sizeof(reloc));
@@ -181,50 +169,34 @@ static void faulting_reloc_and_emit(int fd, drm_intel_bo *target_bo)
 	exec[1].handle = special_bo->handle;
 	exec[1].relocation_count = 1;
 	/* A newly mmap gtt bo will fault on first access. */
-	exec[1].relocs_ptr = (uintptr_t)gtt_relocs;
-	exec[1].alignment = 0;
-	exec[1].offset = 0;
-	exec[1].flags = 0;
-	exec[1].rsvd1 = 0;
-	exec[1].rsvd2 = 0;
+	if (faulting_reloc)
+		exec[1].relocs_ptr = (uintptr_t)gtt_relocs;
+	else
+		exec[1].relocs_ptr = (uintptr_t)reloc;
 
 	execbuf.buffers_ptr = (uintptr_t)exec;
 	execbuf.buffer_count = 2;
-	execbuf.batch_start_offset = 0;
 	execbuf.batch_len = special_batch_len;
-	execbuf.cliprects_ptr = 0;
-	execbuf.num_cliprects = 0;
-	execbuf.DR1 = 0;
-	execbuf.DR4 = 0;
-	execbuf.flags = ring;
-	i915_execbuffer2_set_context_id(execbuf, 0);
-	execbuf.rsvd2 = 0;
+	if (intel_gen(devid) >= 6)
+		execbuf.flags |= I915_EXEC_BLT;
 
 	gem_execbuf(fd, &execbuf);
 
 	gem_close(fd, handle_relocs);
 }
 
-static void reloc_and_emit(int fd, drm_intel_bo *target_bo)
+static struct igt_hang_ring no_hang(int fd)
 {
-	int ring;
-
-	if (intel_gen(devid) >= 6)
-		ring = I915_EXEC_BLT;
-	else
-		ring = 0;
-
-	drm_intel_bo_emit_reloc(special_bo, special_reloc_ofs,
-				target_bo,
-				0,
-				I915_GEM_DOMAIN_RENDER,
-				I915_GEM_DOMAIN_RENDER);
-	drm_intel_bo_mrb_exec(special_bo, special_batch_len, NULL,
-			      0, 0, ring);
-
+	return (struct igt_hang_ring){0};
 }
 
-static void do_test(int fd, bool faulting_reloc)
+static struct igt_hang_ring bcs_hang(int fd)
+{
+	return igt_hang_ring(fd, I915_EXEC_BLT);
+}
+
+static void do_test(int fd, bool faulting_reloc,
+		    struct igt_hang_ring (*do_hang)(int fd))
 {
 	uint32_t tiling_mode = I915_TILING_X;
 	unsigned long pitch, act_size;
@@ -243,14 +215,16 @@ static void do_test(int fd, bool faulting_reloc)
 	create_special_bo();
 
 	for (i = 0; i < NUM_TARGET_BOS; i++) {
+		struct igt_hang_ring hang;
+
 		pc_target_bo[i] = drm_intel_bo_alloc(bufmgr, "special batch", 4096, 4096);
 		emit_dummy_load(pitch);
 		igt_assert(pc_target_bo[i]->offset == 0);
+		hang = do_hang(fd);
 
-		if (faulting_reloc)
-			faulting_reloc_and_emit(fd, pc_target_bo[i]);
-		else
-			reloc_and_emit(fd, pc_target_bo[i]);
+		reloc_and_emit(fd, pc_target_bo[i], faulting_reloc);
+
+		igt_post_hang_ring(fd, hang);
 	}
 
 	/* Only check at the end to avoid unnecessary synchronous behaviour. */
@@ -275,12 +249,16 @@ static void do_test(int fd, bool faulting_reloc)
 #define FAULTING	(1 << 1)
 #define THRASH		(1 << 2)
 #define THRASH_INACTIVE	(1 << 3)
-#define ALL_FLAGS	(INTERRUPT | FAULTING | THRASH | THRASH_INACTIVE)
+#define HANG		(1 << 4)
+#define ALL_FLAGS	(HANG | INTERRUPT | FAULTING | THRASH | THRASH_INACTIVE)
 static void do_forked_test(int fd, unsigned flags)
 {
 	int num_threads = sysconf(_SC_NPROCESSORS_ONLN);
 	struct igt_helper_process thrasher = {};
 
+	if (flags & HANG)
+		igt_require_hang_ring(fd, I915_EXEC_BLT);
+
 	if (flags & (THRASH | THRASH_INACTIVE)) {
 		uint64_t val = (flags & THRASH_INACTIVE) ?
 				(DROP_RETIRE | DROP_BOUND | DROP_UNBOUND) : DROP_ALL;
@@ -295,13 +273,14 @@ static void do_forked_test(int fd, unsigned flags)
 
 	igt_fork(i, num_threads * 4) {
 		/* re-create process local data */
+		fd = drm_open_driver(DRIVER_INTEL);
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		batch = intel_batchbuffer_alloc(bufmgr, devid);
 
 		if (flags & INTERRUPT)
 			igt_fork_signal_helper();
 
-		do_test(fd, flags & FAULTING);
+		do_test(fd, flags & FAULTING, flags & HANG ? bcs_hang : no_hang);
 
 		if (flags & INTERRUPT)
 			igt_stop_signal_helper();
@@ -322,8 +301,7 @@ igt_main
 	memset(blob, 'A', sizeof(blob));
 
 	igt_fixture {
-		fd = drm_open_any();
-
+		fd = drm_open_driver(DRIVER_INTEL);
 		bufmgr = drm_intel_bufmgr_gem_init(fd, 4096);
 		/* disable reuse, otherwise the test fails */
 		//drm_intel_bufmgr_gem_enable_reuse(bufmgr);
@@ -332,28 +310,39 @@ igt_main
 	}
 
 	igt_subtest("normal")
-		do_test(fd, false);
+		do_test(fd, false, no_hang);
 
 	igt_subtest("faulting-reloc")
-		do_test(fd, true);
+		do_test(fd, true, no_hang);
 
 	igt_fork_signal_helper();
 	igt_subtest("interruptible")
-		do_test(fd, false);
+		do_test(fd, false, no_hang);
+
+	igt_subtest("interruptible-hang") {
+		igt_require_hang_ring(fd, I915_EXEC_BLT);
+		do_test(fd, false, bcs_hang);
+	}
 
 	igt_subtest("faulting-reloc-interruptible")
-		do_test(fd, true);
+		do_test(fd, true, no_hang);
+
+	igt_subtest("faulting-reloc-interruptible-hang") {
+		igt_require_hang_ring(fd, I915_EXEC_BLT);
+		do_test(fd, true, bcs_hang);
+	}
 	igt_stop_signal_helper();
 
 	for (unsigned flags = 0; flags <= ALL_FLAGS; flags++) {
 		if ((flags & THRASH) && (flags & THRASH_INACTIVE))
 			continue;
 
-		igt_subtest_f("forked%s%s%s%s",
+		igt_subtest_f("forked%s%s%s%s%s",
 			      flags & INTERRUPT ? "-interruptible" : "",
 			      flags & FAULTING ? "-faulting-reloc" : "",
 			      flags & THRASH ? "-thrashing" : "",
-			      flags & THRASH_INACTIVE ? "-thrash-inactive" : "")
+			      flags & THRASH_INACTIVE ? "-thrash-inactive" : "",
+			      flags & HANG ? "-hang": "")
 			do_forked_test(fd, flags);
 	}
 
