@@ -26,6 +26,8 @@
  */
 
 #define _GNU_SOURCE
+#include "igt.h"
+#include <stdbool.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -39,18 +41,14 @@
 #include <time.h>
 #include <signal.h>
 
-#include "i915_drm.h"
-#include "intel_bufmgr.h"
-#include "intel_batchbuffer.h"
-#include "intel_gpu_tools.h"
-#include "rendercopy.h"
-#include "igt_debugfs.h"
 
 #define RS_NO_ERROR      0
 #define RS_BATCH_ACTIVE  (1 << 0)
 #define RS_BATCH_PENDING (1 << 1)
 #define RS_UNKNOWN       (1 << 2)
 
+static uint32_t devid;
+
 struct local_drm_i915_reset_stats {
 	__u32 ctx_id;
 	__u32 flags;
@@ -60,24 +58,10 @@ struct local_drm_i915_reset_stats {
 	__u32 pad;
 };
 
-struct local_drm_i915_gem_context_create {
-	__u32 ctx_id;
-	__u32 pad;
-};
-
-struct local_drm_i915_gem_context_destroy {
-	__u32 ctx_id;
-	__u32 pad;
-};
-
 #define MAX_FD 32
 
-#define CONTEXT_CREATE_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2d, struct local_drm_i915_gem_context_create)
-#define CONTEXT_DESTROY_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x2e, struct local_drm_i915_gem_context_destroy)
 #define GET_RESET_STATS_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x32, struct local_drm_i915_reset_stats)
 
-static igt_debugfs_t dfs;
-
 #define LOCAL_I915_EXEC_VEBOX	(4 << 0)
 
 struct target_ring;
@@ -87,61 +71,32 @@ static bool gem_has_render(int fd)
 	return true;
 }
 
-static bool has_context(const struct target_ring *ring);
-
 static const struct target_ring {
 	uint32_t exec;
 	bool (*present)(int fd);
-	bool (*contexts)(const struct target_ring *ring);
 	const char *name;
 } rings[] = {
-	{ I915_EXEC_RENDER, gem_has_render, has_context, "render" },
-	{ I915_EXEC_BLT, gem_has_blt, has_context, "blt" },
-	{ I915_EXEC_BSD, gem_has_bsd, has_context, "bsd" },
-	{ LOCAL_I915_EXEC_VEBOX, gem_has_vebox, has_context, "vebox" },
+	{ I915_EXEC_RENDER, gem_has_render, "render" },
+	{ I915_EXEC_BLT, gem_has_blt, "blt" },
+	{ I915_EXEC_BSD, gem_has_bsd, "bsd" },
+	{ LOCAL_I915_EXEC_VEBOX, gem_has_vebox, "vebox" },
 };
 
-static bool has_context(const struct target_ring *ring)
+static void check_context(const struct target_ring *ring)
 {
-	if(ring->exec == I915_EXEC_RENDER)
-		return true;
+	int fd = drm_open_driver(DRIVER_INTEL);
 
-	return false;
+	gem_context_destroy(fd,
+			    gem_context_create(fd));
+	close(fd);
+
+	igt_require(ring->exec == I915_EXEC_RENDER);
 }
 
 #define NUM_RINGS (sizeof(rings)/sizeof(struct target_ring))
 
 static const struct target_ring *current_ring;
 
-static uint32_t context_create(int fd)
-{
-	struct local_drm_i915_gem_context_create create;
-	int ret;
-
-	create.ctx_id = rand();
-	create.pad = rand();
-
-	ret = drmIoctl(fd, CONTEXT_CREATE_IOCTL, &create);
-	igt_assert(ret == 0);
-
-	return create.ctx_id;
-}
-
-static int context_destroy(int fd, uint32_t ctx_id)
-{
-	int ret;
-	struct local_drm_i915_gem_context_destroy destroy;
-
-	destroy.ctx_id = ctx_id;
-	destroy.pad = rand();
-
-	ret = drmIoctl(fd, CONTEXT_DESTROY_IOCTL, &destroy);
-	if (ret != 0)
-		return -errno;
-
-	return 0;
-}
-
 static int gem_reset_stats(int fd, int ctx_id,
 			   struct local_drm_i915_reset_stats *rs)
 {
@@ -237,38 +192,10 @@ static int exec_valid(int fd, int ctx)
 	return exec_valid_ring(fd, ctx, current_ring->exec);
 }
 
-static void stop_rings(const int mask)
-{
-	int fd;
-	char buf[80];
-
-	igt_assert((mask & ~((1 << NUM_RINGS) - 1)) == 0);
-	igt_assert(snprintf(buf, sizeof(buf), "0x%02x", mask) == 4);
-	fd = igt_debugfs_open(&dfs, "i915_ring_stop", O_WRONLY);
-	igt_assert(fd >= 0);
-
-	igt_assert(write(fd, buf, 4) == 4);
-	close(fd);
-}
-
 #define BUFSIZE (4 * 1024)
 #define ITEMS   (BUFSIZE >> 2)
 
-static int ring_to_mask(int ring)
-{
-	for (unsigned i = 0; i < NUM_RINGS; i++) {
-		const struct target_ring *r = &rings[i];
-
-		if (r->exec == ring)
-			return (1 << i);
-	}
-
-	igt_assert(0);
-
-	return -1;
-}
-
-static int inject_hang_ring(int fd, int ctx, int ring)
+static int inject_hang_ring(int fd, int ctx, int ring, bool ignore_ban_error)
 {
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 exec;
@@ -276,10 +203,11 @@ static int inject_hang_ring(int fd, int ctx, int ring)
 	uint32_t *buf;
 	int roff, i;
 	unsigned cmd_len = 2;
+	enum stop_ring_flags flags;
 
 	srandom(time(NULL));
 
-	if (intel_gen(intel_get_drm_devid(fd)) >= 8)
+	if (intel_gen(devid) >= 8)
 		cmd_len = 3;
 
 	buf = malloc(BUFSIZE);
@@ -310,25 +238,25 @@ static int inject_hang_ring(int fd, int ctx, int ring)
 	i915_execbuffer2_set_context_id(execbuf, ctx);
 	execbuf.rsvd2 = 0;
 
-	igt_assert(gem_exec(fd, &execbuf) == 0);
+	igt_assert_eq(gem_exec(fd, &execbuf), 0);
 
 	gtt_off = exec.offset;
 
 	for (i = 0; i < ITEMS; i++)
 		buf[i] = MI_NOOP;
 
-	roff = random() % (ITEMS - cmd_len);
+	roff = random() % (ITEMS - cmd_len - 1);
 	buf[roff] = MI_BATCH_BUFFER_START | (cmd_len - 2);
 	buf[roff + 1] = (gtt_off & 0xfffffffc) + (roff << 2);
 	if (cmd_len == 3)
-		buf[roff + 2] = gtt_off & 0xffffffff00000000ull;
+		buf[roff + 2] = (gtt_off & 0xffffffff00000000ull) >> 32;
 
-#ifdef VERBOSE
-	printf("loop injected at 0x%lx (off 0x%x, bo_start 0x%lx, bo_end 0x%lx)\n",
-	       (long unsigned int)((roff << 2) + gtt_off),
-	       roff << 2, (long unsigned int)gtt_off,
-	       (long unsigned int)(gtt_off + BUFSIZE - 1));
-#endif
+	buf[roff + cmd_len] = MI_BATCH_BUFFER_END;
+
+	igt_debug("loop injected at 0x%lx (off 0x%x, bo_start 0x%lx, bo_end 0x%lx)\n",
+		  (long unsigned int)((roff << 2) + gtt_off),
+		  roff << 2, (long unsigned int)gtt_off,
+		  (long unsigned int)(gtt_off + BUFSIZE - 1));
 	gem_write(fd, exec.handle, 0, buf, BUFSIZE);
 
 	exec.relocation_count = 0;
@@ -351,20 +279,32 @@ static int inject_hang_ring(int fd, int ctx, int ring)
 	i915_execbuffer2_set_context_id(execbuf, ctx);
 	execbuf.rsvd2 = 0;
 
-	igt_assert(gem_exec(fd, &execbuf) == 0);
+	igt_assert_eq(gem_exec(fd, &execbuf), 0);
 
 	igt_assert(gtt_off == exec.offset);
 
 	free(buf);
 
-	stop_rings(ring_to_mask(ring));
+	flags = igt_to_stop_ring_flag(ring);
+
+	flags |= STOP_RING_ALLOW_BAN;
+
+	if (!ignore_ban_error)
+		flags |= STOP_RING_ALLOW_ERRORS;
+
+	igt_set_stop_rings(flags);
 
 	return exec.handle;
 }
 
 static int inject_hang(int fd, int ctx)
 {
-	return inject_hang_ring(fd, ctx, current_ring->exec);
+	return inject_hang_ring(fd, ctx, current_ring->exec, false);
+}
+
+static int inject_hang_no_ban_error(int fd, int ctx)
+{
+	return inject_hang_ring(fd, ctx, current_ring->exec, true);
 }
 
 static int _assert_reset_status(int fd, int ctx, int status)
@@ -373,14 +313,14 @@ static int _assert_reset_status(int fd, int ctx, int status)
 
 	rs = gem_reset_status(fd, ctx);
 	if (rs < 0) {
-		printf("reset status for %d ctx %d returned %d\n",
-		       fd, ctx, rs);
+		igt_info("reset status for %d ctx %d returned %d\n",
+			 fd, ctx, rs);
 		return rs;
 	}
 
 	if (rs != status) {
-		printf("%d:%d reset status %d differs from assumed %d\n",
-		       fd, ctx, rs, status);
+		igt_info("%d:%d reset status %d differs from assumed %d\n",
+			 fd, ctx, rs, status);
 
 		return 1;
 	}
@@ -397,11 +337,11 @@ static void test_rs(int num_fds, int hang_index, int rs_assumed_no_hang)
 	int fd[MAX_FD];
 	int h[MAX_FD];
 
-	igt_assert (num_fds <= MAX_FD);
-	igt_assert (hang_index < MAX_FD);
+	igt_assert_lte(num_fds, MAX_FD);
+	igt_assert_lt(hang_index, MAX_FD);
 
 	for (i = 0; i < num_fds; i++) {
-		fd[i] = drm_open_any();
+		fd[i] = drm_open_driver(DRIVER_INTEL);
 		igt_assert(fd[i]);
 	}
 
@@ -446,21 +386,21 @@ static void test_rs_ctx(int num_fds, int num_ctx, int hang_index,
 	int h[MAX_FD][MAX_CTX];
 	int ctx[MAX_FD][MAX_CTX];
 
-	igt_assert (num_fds <= MAX_FD);
-	igt_assert (hang_index < MAX_FD);
+	igt_assert_lte(num_fds, MAX_FD);
+	igt_assert_lt(hang_index, MAX_FD);
 
-	igt_assert (num_ctx <= MAX_CTX);
-	igt_assert (hang_context < MAX_CTX);
+	igt_assert_lte(num_ctx, MAX_CTX);
+	igt_assert_lt(hang_context, MAX_CTX);
 
 	test_rs(num_fds, -1, RS_NO_ERROR);
 
 	for (i = 0; i < num_fds; i++) {
-		fd[i] = drm_open_any();
+		fd[i] = drm_open_driver(DRIVER_INTEL);
 		igt_assert(fd[i]);
 		assert_reset_status(fd[i], 0, RS_NO_ERROR);
 
 		for (j = 0; j < num_ctx; j++) {
-			ctx[i][j] = context_create(fd[i]);
+			ctx[i][j] = gem_context_create(fd[i]);
 
 		}
 
@@ -512,7 +452,7 @@ static void test_rs_ctx(int num_fds, int num_ctx, int hang_index,
 	for (i = 0; i < num_fds; i++) {
 		for (j = 0; j < num_ctx; j++) {
 			gem_close(fd[i], h[i][j]);
-			igt_assert(context_destroy(fd[i], ctx[i][j]) == 0);
+			gem_context_destroy(fd[i], ctx[i][j]);
 		}
 
 		assert_reset_status(fd[i], 0, RS_NO_ERROR);
@@ -529,25 +469,23 @@ static void test_ban(void)
 	int active_count = 0, pending_count = 0;
 	struct local_drm_i915_reset_stats rs_bad, rs_good;
 
-	fd_bad = drm_open_any();
-	igt_assert(fd_bad >= 0);
+	fd_bad = drm_open_driver(DRIVER_INTEL);
 
-	fd_good = drm_open_any();
-	igt_assert(fd_good >= 0);
+	fd_good = drm_open_driver(DRIVER_INTEL);
 
 	assert_reset_status(fd_bad, 0, RS_NO_ERROR);
 	assert_reset_status(fd_good, 0, RS_NO_ERROR);
 
 	h1 = exec_valid(fd_bad, 0);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	h5 = exec_valid(fd_good, 0);
-	igt_assert(h5 >= 0);
+	igt_assert_lte(0, h5);
 
 	assert_reset_status(fd_bad, 0, RS_NO_ERROR);
 	assert_reset_status(fd_good, 0, RS_NO_ERROR);
 
-	h2 = inject_hang(fd_bad, 0);
-	igt_assert(h2 >= 0);
+	h2 = inject_hang_no_ban_error(fd_bad, 0);
+	igt_assert_lte(0, h2);
 	active_count++;
 	/* Second hang will be pending for this */
 	pending_count++;
@@ -556,8 +494,8 @@ static void test_ban(void)
 	h7 = exec_valid(fd_good, 0);
 
         while (retry--) {
-                h3 = inject_hang(fd_bad, 0);
-                igt_assert(h3 >= 0);
+                h3 = inject_hang_no_ban_error(fd_bad, 0);
+                igt_assert_lte(0, h3);
                 gem_sync(fd_bad, h3);
 		active_count++;
 		/* This second hand will count as pending */
@@ -572,21 +510,21 @@ static void test_ban(void)
                 /* Should not happen often but sometimes hang is declared too slow
                  * due to our way of faking hang using loop */
 
-                igt_assert(h4 >= 0);
+                igt_assert_lte(0, h4);
                 gem_close(fd_bad, h3);
                 gem_close(fd_bad, h4);
 
-                printf("retrying for ban (%d)\n", retry);
+                igt_info("retrying for ban (%d)\n", retry);
         }
 
-	igt_assert(h4 == -EIO);
+	igt_assert_eq(h4, -EIO);
 	assert_reset_status(fd_bad, 0, RS_BATCH_ACTIVE);
 
 	gem_sync(fd_good, h7);
 	assert_reset_status(fd_good, 0, RS_BATCH_PENDING);
 
-	igt_assert(gem_reset_stats(fd_good, 0, &rs_good) == 0);
-	igt_assert(gem_reset_stats(fd_bad, 0, &rs_bad) == 0);
+	igt_assert_eq(gem_reset_stats(fd_good, 0, &rs_good), 0);
+	igt_assert_eq(gem_reset_stats(fd_bad, 0, &rs_bad), 0);
 
 	igt_assert(rs_bad.batch_active == active_count);
 	igt_assert(rs_bad.batch_pending == pending_count);
@@ -599,14 +537,14 @@ static void test_ban(void)
 	gem_close(fd_good, h7);
 
 	h1 = exec_valid(fd_good, 0);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	gem_close(fd_good, h1);
 
 	close(fd_bad);
 	close(fd_good);
 
-	igt_assert(gem_reset_status(fd_bad, 0) < 0);
-	igt_assert(gem_reset_status(fd_good, 0) < 0);
+	igt_assert_lt(gem_reset_status(fd_bad, 0), 0);
+	igt_assert_lt(gem_reset_status(fd_good, 0), 0);
 }
 
 static void test_ban_ctx(void)
@@ -618,28 +556,27 @@ static void test_ban_ctx(void)
 	int active_count = 0, pending_count = 0;
 	struct local_drm_i915_reset_stats rs_bad, rs_good;
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 
-	ctx_good = context_create(fd);
-	ctx_bad = context_create(fd);
+	ctx_good = gem_context_create(fd);
+	ctx_bad = gem_context_create(fd);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 	assert_reset_status(fd, ctx_good, RS_NO_ERROR);
 	assert_reset_status(fd, ctx_bad, RS_NO_ERROR);
 
 	h1 = exec_valid(fd, ctx_bad);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	h5 = exec_valid(fd, ctx_good);
-	igt_assert(h5 >= 0);
+	igt_assert_lte(0, h5);
 
 	assert_reset_status(fd, ctx_good, RS_NO_ERROR);
 	assert_reset_status(fd, ctx_bad, RS_NO_ERROR);
 
-	h2 = inject_hang(fd, ctx_bad);
-	igt_assert(h2 >= 0);
+	h2 = inject_hang_no_ban_error(fd, ctx_bad);
+	igt_assert_lte(0, h2);
 	active_count++;
 	/* Second hang will be pending for this */
 	pending_count++;
@@ -648,8 +585,8 @@ static void test_ban_ctx(void)
 	h7 = exec_valid(fd, ctx_good);
 
         while (retry--) {
-                h3 = inject_hang(fd, ctx_bad);
-                igt_assert(h3 >= 0);
+                h3 = inject_hang_no_ban_error(fd, ctx_bad);
+                igt_assert_lte(0, h3);
                 gem_sync(fd, h3);
 		active_count++;
 		/* This second hand will count as pending */
@@ -664,21 +601,21 @@ static void test_ban_ctx(void)
                 /* Should not happen often but sometimes hang is declared too slow
                  * due to our way of faking hang using loop */
 
-                igt_assert(h4 >= 0);
+                igt_assert_lte(0, h4);
                 gem_close(fd, h3);
                 gem_close(fd, h4);
 
-                printf("retrying for ban (%d)\n", retry);
+                igt_info("retrying for ban (%d)\n", retry);
         }
 
-	igt_assert(h4 == -EIO);
+	igt_assert_eq(h4, -EIO);
 	assert_reset_status(fd, ctx_bad, RS_BATCH_ACTIVE);
 
 	gem_sync(fd, h7);
 	assert_reset_status(fd, ctx_good, RS_BATCH_PENDING);
 
-	igt_assert(gem_reset_stats(fd, ctx_good, &rs_good) == 0);
-	igt_assert(gem_reset_stats(fd, ctx_bad, &rs_bad) == 0);
+	igt_assert_eq(gem_reset_stats(fd, ctx_good, &rs_good), 0);
+	igt_assert_eq(gem_reset_stats(fd, ctx_bad, &rs_bad), 0);
 
 	igt_assert(rs_bad.batch_active == active_count);
 	igt_assert(rs_bad.batch_pending == pending_count);
@@ -691,15 +628,15 @@ static void test_ban_ctx(void)
 	gem_close(fd, h7);
 
 	h1 = exec_valid(fd, ctx_good);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	gem_close(fd, h1);
 
-	igt_assert(context_destroy(fd, ctx_good) == 0);
-	igt_assert(context_destroy(fd, ctx_bad) == 0);
-	igt_assert(gem_reset_status(fd, ctx_good) < 0);
-	igt_assert(gem_reset_status(fd, ctx_bad) < 0);
-	igt_assert(exec_valid(fd, ctx_good) < 0);
-	igt_assert(exec_valid(fd, ctx_bad) < 0);
+	gem_context_destroy(fd, ctx_good);
+	gem_context_destroy(fd, ctx_bad);
+	igt_assert_lt(gem_reset_status(fd, ctx_good), 0);
+	igt_assert_lt(gem_reset_status(fd, ctx_bad), 0);
+	igt_assert_lt(exec_valid(fd, ctx_good), 0);
+	igt_assert_lt(exec_valid(fd, ctx_bad), 0);
 
 	close(fd);
 }
@@ -710,32 +647,32 @@ static void test_unrelated_ctx(void)
 	int fd1,fd2;
 	int ctx_guilty, ctx_unrelated;
 
-	fd1 = drm_open_any();
-	fd2 = drm_open_any();
+	fd1 = drm_open_driver(DRIVER_INTEL);
+	fd2 = drm_open_driver(DRIVER_INTEL);
 	assert_reset_status(fd1, 0, RS_NO_ERROR);
 	assert_reset_status(fd2, 0, RS_NO_ERROR);
-	ctx_guilty = context_create(fd1);
-	ctx_unrelated = context_create(fd2);
+	ctx_guilty = gem_context_create(fd1);
+	ctx_unrelated = gem_context_create(fd2);
 
 	assert_reset_status(fd1, ctx_guilty, RS_NO_ERROR);
 	assert_reset_status(fd2, ctx_unrelated, RS_NO_ERROR);
 
 	h1 = inject_hang(fd1, ctx_guilty);
-	igt_assert(h1 >= 0);
+	igt_assert_lte(0, h1);
 	gem_sync(fd1, h1);
 	assert_reset_status(fd1, ctx_guilty, RS_BATCH_ACTIVE);
 	assert_reset_status(fd2, ctx_unrelated, RS_NO_ERROR);
 
 	h2 = exec_valid(fd2, ctx_unrelated);
-	igt_assert(h2 >= 0);
+	igt_assert_lte(0, h2);
 	gem_sync(fd2, h2);
 	assert_reset_status(fd1, ctx_guilty, RS_BATCH_ACTIVE);
 	assert_reset_status(fd2, ctx_unrelated, RS_NO_ERROR);
 	gem_close(fd1, h1);
 	gem_close(fd2, h2);
 
-	igt_assert(context_destroy(fd1, ctx_guilty) == 0);
-	igt_assert(context_destroy(fd2, ctx_unrelated) == 0);
+	gem_context_destroy(fd1, ctx_guilty);
+	gem_context_destroy(fd2, ctx_unrelated);
 
 	close(fd1);
 	close(fd2);
@@ -758,16 +695,15 @@ static void test_close_pending_ctx(void)
 	int fd, h;
 	uint32_t ctx;
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
-	ctx = context_create(fd);
+	fd = drm_open_driver(DRIVER_INTEL);
+	ctx = gem_context_create(fd);
 
 	assert_reset_status(fd, ctx, RS_NO_ERROR);
 
 	h = inject_hang(fd, ctx);
-	igt_assert(h >= 0);
-	igt_assert(context_destroy(fd, ctx) == 0);
-	igt_assert(context_destroy(fd, ctx) == -ENOENT);
+	igt_assert_lte(0, h);
+	gem_context_destroy(fd, ctx);
+	igt_assert(__gem_context_destroy(fd, ctx) == -ENOENT);
 
 	gem_close(fd, h);
 	close(fd);
@@ -777,13 +713,12 @@ static void test_close_pending(void)
 {
 	int fd, h;
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 
 	h = inject_hang(fd, 0);
-	igt_assert(h >= 0);
+	igt_assert_lte(0, h);
 
 	gem_close(fd, h);
 	close(fd);
@@ -840,13 +775,12 @@ static void test_close_pending_fork(const bool reverse)
 	int pid;
 	int fd, h;
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	assert_reset_status(fd, 0, RS_NO_ERROR);
 
 	h = inject_hang(fd, 0);
-	igt_assert(h >= 0);
+	igt_assert_lte(0, h);
 
 	sleep(1);
 
@@ -856,8 +790,8 @@ static void test_close_pending_fork(const bool reverse)
 	 */
 	pid = fork();
 	if (pid == 0) {
-		const int fd2 = drm_open_any();
-		igt_assert(fd2 >= 0);
+		const int fd2 = drm_open_driver(DRIVER_INTEL);
+		igt_assert_lte(0, fd2);
 
 		/* The crucial component is that we schedule the same noop batch
 		 * on each ring. This exercises batch_obj reference counting,
@@ -868,7 +802,7 @@ static void test_close_pending_fork(const bool reverse)
 		close(fd2);
 		return;
 	} else {
-		igt_assert(pid > 0);
+		igt_assert_lt(0, pid);
 		sleep(1);
 
 		/* Kill the child to reduce refcounts on
@@ -880,11 +814,10 @@ static void test_close_pending_fork(const bool reverse)
 	close(fd);
 
 	/* Then we just wait on hang to happen */
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 
 	h = exec_valid(fd, 0);
-	igt_assert(h >= 0);
+	igt_assert_lte(0, h);
 
 	gem_sync(fd, h);
 	gem_close(fd, h);
@@ -896,10 +829,9 @@ static void test_reset_count(const bool create_ctx)
 	int fd, h, ctx;
 	long c1, c2;
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
+	fd = drm_open_driver(DRIVER_INTEL);
 	if (create_ctx)
-		ctx = context_create(fd);
+		ctx = gem_context_create(fd);
 	else
 		ctx = 0;
 
@@ -909,7 +841,7 @@ static void test_reset_count(const bool create_ctx)
 	igt_assert(c1 >= 0);
 
 	h = inject_hang(fd, ctx);
-	igt_assert (h >= 0);
+	igt_assert_lte(0, h);
 	gem_sync(fd, h);
 
 	assert_reset_status(fd, ctx, RS_BATCH_ACTIVE);
@@ -933,7 +865,7 @@ static void test_reset_count(const bool create_ctx)
 	gem_close(fd, h);
 
 	if (create_ctx)
-		context_destroy(fd, ctx);
+		gem_context_destroy(fd, ctx);
 
 	close(fd);
 }
@@ -962,29 +894,28 @@ static int _test_params(int fd, int ctx, uint32_t flags, uint32_t pad)
 
 typedef enum { root = 0, user } cap_t;
 
-static void test_param_ctx(const int fd, const int ctx, const cap_t cap)
+static void _check_param_ctx(const int fd, const int ctx, const cap_t cap)
 {
 	const uint32_t bad = rand() + 1;
 
 	if (ctx == 0) {
 		if (cap == root)
-			igt_assert(_test_params(fd, ctx, 0, 0) == 0);
+			igt_assert_eq(_test_params(fd, ctx, 0, 0), 0);
 		else
-			igt_assert(_test_params(fd, ctx, 0, 0) == -EPERM);
+			igt_assert_eq(_test_params(fd, ctx, 0, 0), -EPERM);
 	}
 
-	igt_assert(_test_params(fd, ctx, 0, bad) == -EINVAL);
-	igt_assert(_test_params(fd, ctx, bad, 0) == -EINVAL);
-	igt_assert(_test_params(fd, ctx, bad, bad) == -EINVAL);
+	igt_assert_eq(_test_params(fd, ctx, 0, bad), -EINVAL);
+	igt_assert_eq(_test_params(fd, ctx, bad, 0), -EINVAL);
+	igt_assert_eq(_test_params(fd, ctx, bad, bad), -EINVAL);
 }
 
 static void check_params(const int fd, const int ctx, cap_t cap)
 {
 	igt_assert(ioctl(fd, GET_RESET_STATS_IOCTL, 0) == -1);
-	igt_assert(_test_params(fd, 0xbadbad, 0, 0) == -ENOENT);
+	igt_assert_eq(_test_params(fd, 0xbadbad, 0, 0), -ENOENT);
 
-	test_param_ctx(fd, 0, cap);
-	test_param_ctx(fd, ctx, cap);
+	_check_param_ctx(fd, ctx, cap);
 }
 
 static void _test_param(const int fd, const int ctx)
@@ -1004,44 +935,140 @@ static void _test_param(const int fd, const int ctx)
 	igt_waitchildren();
 }
 
-static void test_params(void)
+static void test_params_ctx(void)
 {
 	int fd, ctx;
 
-	fd = drm_open_any();
-	igt_assert(fd >= 0);
-	ctx = context_create(fd);
+	fd = drm_open_driver(DRIVER_INTEL);
+	ctx = gem_context_create(fd);
 
 	_test_param(fd, ctx);
 
 	close(fd);
 }
 
-#define RING_HAS_CONTEXTS current_ring->contexts(current_ring)
-#define RUN_CTX_TEST(...) do { igt_skip_on(RING_HAS_CONTEXTS == false); __VA_ARGS__; } while (0)
+static void test_params(void)
+{
+	int fd;
 
-int fd;
+	fd = drm_open_driver(DRIVER_INTEL);
+
+	_test_param(fd, 0);
+
+	close(fd);
+
+}
+
+static void defer_hangcheck(int ring_num)
+{
+	int fd, count_start, count_end;
+	int seconds = 30;
+	const struct target_ring *next_ring;
+	fd = drm_open_driver(DRIVER_INTEL);
+
+	do {
+		next_ring = &rings[(++ring_num) % NUM_RINGS];
+
+		if (next_ring->present(fd))
+			break;
+
+	} while(next_ring != current_ring);
+
+	igt_skip_on(next_ring == current_ring);
+
+	count_start = get_reset_count(fd, 0);
+	igt_assert_lte(0, count_start);
+
+	igt_assert(inject_hang_ring(fd, 0, current_ring->exec, true));
+	while (--seconds) {
+		igt_assert(exec_valid_ring(fd, 0, next_ring->exec));
+
+		count_end = get_reset_count(fd, 0);
+		igt_assert_lte(0, count_end);
+
+		if (count_end > count_start)
+			break;
+
+		sleep(1);
+	}
+
+	igt_assert_lt(count_start, count_end);
+
+	close(fd);
+}
+
+static bool gem_has_reset_stats(int fd)
+{
+	struct local_drm_i915_reset_stats rs;
+	int ret;
+
+	/* Carefully set flags and pad to zero, otherwise
+	   we get -EINVAL
+	*/
+	memset(&rs, 0, sizeof(rs));
+
+	ret = drmIoctl(fd, GET_RESET_STATS_IOCTL, &rs);
+	if (ret == 0)
+		return true;
+
+	/* If we get EPERM, we have support but did not
+	   have CAP_SYSADM */
+	if (ret == -1 && errno == EPERM)
+		return true;
+
+	return false;
+}
+
+static void check_gpu_ok(void)
+{
+	int retry_count = 30;
+	enum stop_ring_flags flags;
+	int fd;
+
+	igt_debug("checking gpu state\n");
+
+	while (retry_count--) {
+		flags = igt_get_stop_rings() & STOP_RING_ALL;
+		if (flags == 0)
+			break;
+
+		igt_debug("waiting previous hang to clear\n");
+		sleep(1);
+	}
+
+	igt_assert(flags == 0);
+
+	/*
+	 * Clear the _ALLOW_ERRORS and _ALLOW_BAN flags;
+	 * these are not cleared by individual ring reset.
+	 */
+	igt_set_stop_rings(0);
+
+	fd = drm_open_driver(DRIVER_INTEL);
+	gem_quiescent_gpu(fd);
+	close(fd);
+}
+
+#define RUN_TEST(...) do { check_gpu_ok(); __VA_ARGS__; check_gpu_ok(); } while (0)
+#define RUN_CTX_TEST(...) do { check_context(current_ring); RUN_TEST(__VA_ARGS__); } while (0)
 
 igt_main
 {
-	struct local_drm_i915_gem_context_create create;
-	uint32_t devid;
-	int ret;
-
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		fd = drm_open_any();
+		int fd;
+
+		bool has_reset_stats;
+		fd = drm_open_driver(DRIVER_INTEL);
 		devid = intel_get_drm_devid(fd);
-		igt_require_f(intel_gen(devid) >= 4,
-			      "Architecture %d too old\n", intel_gen(devid));
 
-		ret = drmIoctl(fd, CONTEXT_CREATE_IOCTL, &create);
-		igt_skip_on_f(ret != 0 && (errno == ENODEV || errno == EINVAL),
-			      "Kernel is too old, or contexts not supported: %s\n",
-			      strerror(errno));
+		has_reset_stats = gem_has_reset_stats(fd);
 
-		igt_debugfs_init(&dfs);
+		close(fd);
+
+		igt_require_f(has_reset_stats,
+			      "No reset stats ioctl support. Too old kernel?\n");
 	}
 
 	igt_subtest("params")
@@ -1053,23 +1080,33 @@ igt_main
 		current_ring = &rings[i];
 		name = current_ring->name;
 
-		igt_fixture
+		igt_fixture {
+			int fd = drm_open_driver(DRIVER_INTEL);
 			gem_require_ring(fd, current_ring->exec);
+			close(fd);
+		}
+
+		igt_fixture
+			igt_require_f(intel_gen(devid) >= 4,
+				      "gen %d doesn't support reset\n", intel_gen(devid));
+
+		igt_subtest_f("params-ctx-%s", name)
+			RUN_CTX_TEST(test_params_ctx());
 
 		igt_subtest_f("reset-stats-%s", name)
-			test_rs(4, 1, 0);
+			RUN_TEST(test_rs(4, 1, 0));
 
 		igt_subtest_f("reset-stats-ctx-%s", name)
 			RUN_CTX_TEST(test_rs_ctx(4, 4, 1, 2));
 
 		igt_subtest_f("ban-%s", name)
-			test_ban();
+			RUN_TEST(test_ban());
 
 		igt_subtest_f("ban-ctx-%s", name)
 			RUN_CTX_TEST(test_ban_ctx());
 
 		igt_subtest_f("reset-count-%s", name)
-			test_reset_count(false);
+			RUN_TEST(test_reset_count(false));
 
 		igt_subtest_f("reset-count-ctx-%s", name)
 			RUN_CTX_TEST(test_reset_count(true));
@@ -1077,22 +1114,20 @@ igt_main
 		igt_subtest_f("unrelated-ctx-%s", name)
 			RUN_CTX_TEST(test_unrelated_ctx());
 
-		igt_subtest_f("close-pending-%s", name) {
-			test_close_pending();
-			gem_quiescent_gpu(fd);
-		}
+		igt_subtest_f("close-pending-%s", name)
+			RUN_TEST(test_close_pending());
 
-		igt_subtest_f("close-pending-ctx-%s", name) {
+		igt_subtest_f("close-pending-ctx-%s", name)
 			RUN_CTX_TEST(test_close_pending_ctx());
-			gem_quiescent_gpu(fd);
-		}
 
-		igt_subtest_f("close-pending-fork-%s", name) {
-			test_close_pending_fork(true);
-			test_close_pending_fork(false);
-		}
+		igt_subtest_f("close-pending-fork-%s", name)
+			RUN_TEST(test_close_pending_fork(false));
+
+		igt_subtest_f("close-pending-fork-reverse-%s", name)
+			RUN_TEST(test_close_pending_fork(true));
+
+		igt_subtest_f("defer-hangcheck-%s", name)
+			RUN_TEST(defer_hangcheck(i));
+
 	}
-
-	igt_fixture
-		close(fd);
 }
