@@ -22,17 +22,12 @@
  *
  */
 
+#include "igt.h"
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
 
-#include "drmtest.h"
-#include "igt_debugfs.h"
-#include "igt_kms.h"
-#include "intel_chipset.h"
-#include "intel_batchbuffer.h"
-#include "ioctl_wrappers.h"
 
 IGT_TEST_DESCRIPTION(
    "Performs various write operations to the scanout buffer while FBC is "
@@ -55,18 +50,16 @@ enum test_mode {
 
 typedef struct {
 	int drm_fd;
-	igt_crc_t ref_crc[2];
+	igt_crc_t ref_crc[4];
 	igt_pipe_crc_t *pipe_crc;
 	drm_intel_bufmgr *bufmgr;
 	drm_intel_context *ctx[2];
 	uint32_t devid;
-	uint32_t handle[2];
 	igt_display_t display;
 	igt_output_t *output;
 	enum pipe pipe;
 	igt_plane_t *primary;
 	struct igt_fb fb[2];
-	uint32_t fb_id[2];
 } data_t;
 
 static const char *test_mode_str(enum test_mode mode)
@@ -99,6 +92,7 @@ static void fill_blt(data_t *data,
 	struct intel_batchbuffer *batch;
 	unsigned flags;
 	int pitch;
+	uint32_t pixel = color | (color << 8) | (color << 16) | (color << 24);
 
 	batch = intel_batchbuffer_alloc(data->bufmgr, data->devid);
 	igt_assert(batch);
@@ -114,9 +108,9 @@ static void fill_blt(data_t *data,
 	COLOR_BLIT_COPY_BATCH_START(flags);
 	OUT_BATCH(3 << 24 | 0xf0 << 16 | pitch);
 	OUT_BATCH(0);
-	OUT_BATCH(fb->height << 16 | fb->width);
+	OUT_BATCH(1 << 16 | 1);
 	OUT_RELOC_FENCED(dst, I915_GEM_DOMAIN_RENDER, I915_GEM_DOMAIN_RENDER, 0);
-	OUT_BATCH(color);
+	OUT_BATCH(pixel);
 	ADVANCE_BATCH();
 
 	intel_batchbuffer_flush(batch);
@@ -190,69 +184,111 @@ static void fill_render(data_t *data, uint32_t handle,
 	gem_bo_busy(data->drm_fd, handle);
 }
 
+static void fill_mmap_cpu(data_t *data, uint32_t handle, unsigned char color)
+{
+	void *ptr;
+
+	ptr = gem_mmap__cpu(data->drm_fd, handle, 0, 4096, PROT_WRITE);
+	gem_set_domain(data->drm_fd, handle, I915_GEM_DOMAIN_CPU,
+		       I915_GEM_DOMAIN_CPU);
+	memset(ptr, color, 4);
+	munmap(ptr, 4096);
+	gem_sw_finish(data->drm_fd, handle);
+}
+
+static void fill_mmap_gtt(data_t *data, uint32_t handle, unsigned char color)
+{
+	void *ptr;
+
+	ptr = gem_mmap__gtt(data->drm_fd, handle, 4096, PROT_WRITE);
+	gem_set_domain(data->drm_fd, handle, I915_GEM_DOMAIN_GTT,
+		       I915_GEM_DOMAIN_GTT);
+	memset(ptr, color, 4);
+	munmap(ptr, 4096);
+}
+
 static bool fbc_enabled(data_t *data)
 {
-	FILE *status;
-	char str[64] = {};
+	char str[128] = {};
 
-	status = igt_debugfs_fopen("i915_fbc_status", "r");
-	igt_assert(status);
-
-	fread(str, sizeof(str) - 1, 1, status);
-	fclose(status);
+	igt_debugfs_read("i915_fbc_status", str);
 	return strstr(str, "FBC enabled") != NULL;
 }
 
+static bool wait_for_fbc_enabled(data_t *data)
+{
+	return igt_wait(fbc_enabled(data), 3000, 30);
+}
+
+static void check_crc(data_t *data, enum test_mode mode)
+{
+	igt_pipe_crc_t *pipe_crc = data->pipe_crc;
+	igt_crc_t crc, *ref_crc;
+
+	switch (mode) {
+	case TEST_PAGE_FLIP:
+		ref_crc = &data->ref_crc[1];
+		break;
+	case TEST_MMAP_CPU:
+	case TEST_MMAP_GTT:
+	case TEST_BLT:
+	case TEST_RENDER:
+	case TEST_CONTEXT:
+		ref_crc = &data->ref_crc[2];
+		break;
+	case TEST_PAGE_FLIP_AND_MMAP_CPU:
+	case TEST_PAGE_FLIP_AND_MMAP_GTT:
+	case TEST_PAGE_FLIP_AND_BLT:
+	case TEST_PAGE_FLIP_AND_RENDER:
+	case TEST_PAGE_FLIP_AND_CONTEXT:
+		ref_crc = &data->ref_crc[3];
+		break;
+	default:
+		igt_assert(false);
+	}
+
+	igt_pipe_crc_collect_crc(pipe_crc, &crc);
+	igt_assert_crc_equal(&crc, ref_crc);
+}
+
 static void test_crc(data_t *data, enum test_mode mode)
 {
 	uint32_t crtc_id = data->output->config.crtc->crtc_id;
-	igt_pipe_crc_t *pipe_crc = data->pipe_crc;
-	igt_crc_t *crcs = NULL;
-	uint32_t handle = data->handle[0];
+	uint32_t handle = data->fb[0].gem_handle;
 
 	igt_assert(fbc_enabled(data));
 
-	if (mode >= TEST_PAGE_FLIP_AND_MMAP_CPU) {
-		handle = data->handle[1];
+	if (mode == TEST_PAGE_FLIP || mode >= TEST_PAGE_FLIP_AND_MMAP_CPU) {
+		handle = data->fb[1].gem_handle;
 		igt_assert(drmModePageFlip(data->drm_fd, crtc_id,
-					   data->fb_id[1], 0, NULL) == 0);
-		usleep(300000);
+					   data->fb[1].fb_id, 0, NULL) == 0);
 
-		igt_assert(fbc_enabled(data));
+		if (mode != TEST_PAGE_FLIP)
+			igt_assert(wait_for_fbc_enabled(data));
 	}
 
 	switch (mode) {
-		void *ptr;
+		drm_intel_context *context = NULL;
 	case TEST_PAGE_FLIP:
-		igt_assert(drmModePageFlip(data->drm_fd, crtc_id,
-					   data->fb_id[1], 0, NULL) == 0);
 		break;
 	case TEST_MMAP_CPU:
 	case TEST_PAGE_FLIP_AND_MMAP_CPU:
-		ptr = gem_mmap__cpu(data->drm_fd, handle, 0, 4096, PROT_WRITE);
-		gem_set_domain(data->drm_fd, handle, I915_GEM_DOMAIN_CPU, I915_GEM_DOMAIN_CPU);
-		memset(ptr, 0xff, 4);
-		munmap(ptr, 4096);
-		gem_sw_finish(data->drm_fd, handle);
+		fill_mmap_cpu(data, handle, 0xff);
 		break;
 	case TEST_MMAP_GTT:
 	case TEST_PAGE_FLIP_AND_MMAP_GTT:
-		ptr = gem_mmap__gtt(data->drm_fd, handle, 4096, PROT_WRITE);
-		gem_set_domain(data->drm_fd, handle, I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
-		memset(ptr, 0xff, 4);
-		munmap(ptr, 4096);
+		fill_mmap_gtt(data, handle, 0xff);
 		break;
 	case TEST_BLT:
 	case TEST_PAGE_FLIP_AND_BLT:
 		fill_blt(data, handle, data->fb, ~0);
 		break;
-	case TEST_RENDER:
 	case TEST_CONTEXT:
-	case TEST_PAGE_FLIP_AND_RENDER:
 	case TEST_PAGE_FLIP_AND_CONTEXT:
-		fill_render(data, handle,
-			    (mode == TEST_CONTEXT || mode == TEST_PAGE_FLIP_AND_CONTEXT) ?
-			    data->ctx[1] : NULL, 0xff);
+		context = data->ctx[1];
+	case TEST_RENDER:
+	case TEST_PAGE_FLIP_AND_RENDER:
+		fill_render(data, handle, context, 0xff);
 		break;
 	}
 
@@ -264,33 +300,15 @@ static void test_crc(data_t *data, enum test_mode mode)
 	igt_wait_for_vblank(data->drm_fd, data->pipe);
 	igt_wait_for_vblank(data->drm_fd, data->pipe);
 
-	igt_pipe_crc_start(pipe_crc);
-	igt_pipe_crc_get_crcs(pipe_crc, 1, &crcs);
-	igt_pipe_crc_stop(pipe_crc);
-	igt_assert(!igt_crc_equal(&crcs[0], &data->ref_crc[0]));
-	if (mode == TEST_PAGE_FLIP)
-		igt_assert(igt_crc_equal(&crcs[0], &data->ref_crc[1]));
-	else
-		igt_assert(!igt_crc_equal(&crcs[0], &data->ref_crc[1]));
-	free(crcs);
+	check_crc(data, mode);
 
 	/*
 	 * Allow time for FBC to kick in again if it
 	 * got disabled during dirtyfb or page flip.
 	 */
-	usleep(300000);
+	igt_assert(wait_for_fbc_enabled(data));
 
-	igt_assert(fbc_enabled(data));
-
-	igt_pipe_crc_start(pipe_crc);
-	igt_pipe_crc_get_crcs(pipe_crc, 1, &crcs);
-	igt_pipe_crc_stop(pipe_crc);
-	igt_assert(!igt_crc_equal(&crcs[0], &data->ref_crc[0]));
-	if (mode == TEST_PAGE_FLIP)
-		igt_assert(igt_crc_equal(&crcs[0], &data->ref_crc[1]));
-	else
-		igt_assert(!igt_crc_equal(&crcs[0], &data->ref_crc[1]));
-	free(crcs);
+	check_crc(data, mode);
 }
 
 static bool prepare_crtc(data_t *data)
@@ -311,38 +329,77 @@ static bool prepare_crtc(data_t *data)
 	return true;
 }
 
+static void create_fbs(data_t *data, bool tiled, struct igt_fb *fbs)
+{
+	int rc;
+	drmModeModeInfo *mode = igt_output_get_mode(data->output);
+	uint64_t tiling = tiled ? LOCAL_I915_FORMAT_MOD_X_TILED :
+				  LOCAL_DRM_FORMAT_MOD_NONE;
+
+	rc = igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
+				 DRM_FORMAT_XRGB8888, tiling,
+				 0.0, 0.0, 0.0, &fbs[0]);
+	igt_assert(rc);
+	rc = igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
+				 DRM_FORMAT_XRGB8888, tiling,
+				 0.1, 0.1, 0.1, &fbs[1]);
+	igt_assert(rc);
+}
+
+/* Since we want to be really safe that the CRCs are actually what we really
+ * want, use untiled FBs, so FBC won't happen to disrupt things. Also do the
+ * drawing before setting the modes, just to be sure. */
+static void get_ref_crcs(data_t *data)
+{
+	igt_display_t *display = &data->display;
+	struct igt_fb fbs[4];
+	int i;
+
+	create_fbs(data, false, &fbs[0]);
+	create_fbs(data, false, &fbs[2]);
+
+	fill_mmap_gtt(data, fbs[2].gem_handle, 0xff);
+	fill_mmap_gtt(data, fbs[3].gem_handle, 0xff);
+
+	for (i = 0; i < 4; i++) {
+		igt_plane_set_fb(data->primary, &fbs[i]);
+		igt_display_commit(display);
+		igt_wait_for_vblank(data->drm_fd, data->pipe);
+		igt_assert(!fbc_enabled(data));
+		igt_pipe_crc_collect_crc(data->pipe_crc, &data->ref_crc[i]);
+		igt_assert(!fbc_enabled(data));
+	}
+
+	igt_plane_set_fb(data->primary, &data->fb[1]);
+	igt_display_commit(display);
+
+	for (i = 0; i < 4; i++)
+		igt_remove_fb(data->drm_fd, &fbs[i]);
+}
+
 static bool prepare_test(data_t *data, enum test_mode test_mode)
 {
 	igt_display_t *display = &data->display;
 	igt_output_t *output = data->output;
-	drmModeModeInfo *mode;
 	igt_pipe_crc_t *pipe_crc;
-	igt_crc_t *crcs = NULL;
 
 	data->primary = igt_output_get_plane(data->output, IGT_PLANE_PRIMARY);
-	mode = igt_output_get_mode(data->output);
 
-	data->fb_id[0] = igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
-					     DRM_FORMAT_XRGB8888,
-					     LOCAL_I915_FORMAT_MOD_X_TILED,
-					     0.0, 0.0, 0.0, &data->fb[0]);
-	igt_assert(data->fb_id[0]);
-	data->fb_id[1] = igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
-					     DRM_FORMAT_XRGB8888,
-					     LOCAL_I915_FORMAT_MOD_X_TILED,
-					     0.1, 0.1, 0.1,
-					     &data->fb[1]);
-	igt_assert(data->fb_id[1]);
+	create_fbs(data, true, data->fb);
 
-	data->handle[0] = data->fb[0].gem_handle;
-	data->handle[1] = data->fb[1].gem_handle;
+	igt_pipe_crc_free(data->pipe_crc);
+	data->pipe_crc = NULL;
+	pipe_crc = igt_pipe_crc_new(data->pipe,
+				    INTEL_PIPE_CRC_SOURCE_AUTO);
+	data->pipe_crc = pipe_crc;
+
+	get_ref_crcs(data);
 
 	/* scanout = fb[1] */
 	igt_plane_set_fb(data->primary, &data->fb[1]);
 	igt_display_commit(display);
-	usleep(300000);
 
-	if (!fbc_enabled(data)) {
+	if (!wait_for_fbc_enabled(data)) {
 		igt_info("FBC not enabled\n");
 
 		igt_plane_set_fb(data->primary, NULL);
@@ -354,23 +411,6 @@ static bool prepare_test(data_t *data, enum test_mode test_mode)
 		return false;
 	}
 
-	igt_pipe_crc_free(data->pipe_crc);
-	data->pipe_crc = NULL;
-
-	pipe_crc = igt_pipe_crc_new(data->pipe,
-				    INTEL_PIPE_CRC_SOURCE_AUTO);
-
-	data->pipe_crc = pipe_crc;
-
-	igt_wait_for_vblank(data->drm_fd, data->pipe);
-
-	/* get reference crc for fb[1] */
-	igt_pipe_crc_start(pipe_crc);
-	igt_pipe_crc_get_crcs(pipe_crc, 1, &crcs);
-	data->ref_crc[1] = crcs[0];
-	igt_pipe_crc_stop(pipe_crc);
-	free(crcs);
-
 	if (test_mode == TEST_CONTEXT || test_mode == TEST_PAGE_FLIP_AND_CONTEXT) {
 		data->ctx[0] = drm_intel_gem_context_create(data->bufmgr);
 		igt_assert(data->ctx[0]);
@@ -381,18 +421,17 @@ static bool prepare_test(data_t *data, enum test_mode test_mode)
 		 * Disable FBC RT address for both contexts
 		 * (by "rendering" to a non-scanout buffer).
 		 */
-		exec_nop(data, data->handle[0], data->ctx[1]);
-		exec_nop(data, data->handle[0], data->ctx[0]);
-		exec_nop(data, data->handle[0], data->ctx[1]);
-		exec_nop(data, data->handle[0], data->ctx[0]);
+		exec_nop(data, data->fb[0].gem_handle, data->ctx[1]);
+		exec_nop(data, data->fb[0].gem_handle, data->ctx[0]);
+		exec_nop(data, data->fb[0].gem_handle, data->ctx[1]);
+		exec_nop(data, data->fb[0].gem_handle, data->ctx[0]);
 	}
 
 	/* scanout = fb[0] */
 	igt_plane_set_fb(data->primary, &data->fb[0]);
 	igt_display_commit(display);
-	usleep(300000);
 
-	igt_assert(fbc_enabled(data));
+	igt_assert(wait_for_fbc_enabled(data));
 
 	if (test_mode == TEST_CONTEXT || test_mode == TEST_PAGE_FLIP_AND_CONTEXT) {
 		/*
@@ -400,17 +439,11 @@ static bool prepare_test(data_t *data, enum test_mode test_mode)
 		 * FBC RT address is left as disabled.
 		 */
 		exec_nop(data, data->fb[0].gem_handle, data->ctx[0]);
+		igt_assert(wait_for_fbc_enabled(data));
 	}
 
 	igt_wait_for_vblank(data->drm_fd, data->pipe);
 
-	/* get reference crc for fb[0] */
-	igt_pipe_crc_start(pipe_crc);
-	igt_pipe_crc_get_crcs(pipe_crc, 1, &crcs);
-	data->ref_crc[0] = crcs[0];
-	igt_pipe_crc_stop(pipe_crc);
-	free(crcs);
-
 	return true;
 }
 
@@ -500,25 +533,21 @@ igt_main
 	igt_skip_on_simulation();
 
 	igt_fixture {
-		char buf[64];
-		FILE *status;
+		char buf[128];
 
-		data.drm_fd = drm_open_any_master();
+		data.drm_fd = drm_open_driver_master(DRIVER_INTEL);
 		kmstest_set_vt_graphics_mode();
 
 		data.devid = intel_get_drm_devid(data.drm_fd);
 
 		igt_require_pipe_crc();
 
-		status = igt_debugfs_fopen("i915_fbc_status", "r");
-		igt_require_f(status, "No i915_fbc_status found\n");
-		fread(buf, sizeof(buf), 1, status);
-		fclose(status);
-		buf[sizeof(buf) - 1] = '\0';
-		igt_require_f(!strstr(buf, "unsupported by this chipset") &&
-			      !strstr(buf, "disabled per module param") &&
-			      !strstr(buf, "disabled per chip default"),
-			      "FBC not supported/enabled\n");
+		igt_debugfs_read("i915_fbc_status", buf);
+		igt_require_f(!strstr(buf, "unsupported on this chipset"),
+			      "FBC not supported\n");
+
+		if (intel_gen(data.devid) >= 6)
+			igt_set_module_param_int("enable_fbc", 1);
 
 		data.bufmgr = drm_intel_bufmgr_gem_init(data.drm_fd, 4096);
 		igt_assert(data.bufmgr);
