@@ -24,15 +24,12 @@
  *
  */
 
+#include "igt.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 
-#include "drmtest.h"
-#include "igt_kms.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
 
 /* We know that if we don't enable audio runtime PM, snd_hda_intel will never
  * release its power well refcount, and we'll never reach the LPSP sate. OTOH
@@ -45,12 +42,12 @@ static void disable_audio_runtime_pm(void)
 
 	fd = open("/sys/module/snd_hda_intel/parameters/power_save", O_WRONLY);
 	if (fd >= 0) {
-		write(fd, "1\n", 2);
+		igt_assert_eq(write(fd, "1\n", 2), 2);
 		close(fd);
 	}
 	fd = open("/sys/bus/pci/devices/0000:00:03.0/power/control", O_WRONLY);
 	if (fd >= 0) {
-		write(fd, "auto\n", 5);
+		igt_assert_eq(write(fd, "auto\n", 5), 5);
 		close(fd);
 	}
 	/* Give some time for it to react. */
@@ -226,7 +223,7 @@ igt_main
 	igt_fixture {
 		int i;
 
-		drm_fd = drm_open_any_master();
+		drm_fd = drm_open_driver_master(DRIVER_INTEL);
 		igt_require(drm_fd >= 0);
 
 		devid = intel_get_drm_devid(drm_fd);
