@@ -25,6 +25,7 @@
  *
  */
 
+#include "igt.h"
 #include <stdio.h>
 #include <stdint.h>
 #include <stdbool.h>
@@ -42,17 +43,6 @@
 
 #include <drm.h>
 
-#include "drmtest.h"
-#include "intel_batchbuffer.h"
-#include "intel_io.h"
-#include "intel_chipset.h"
-#include "ioctl_wrappers.h"
-#include "igt_aux.h"
-#include "igt_kms.h"
-#include "igt_gt.h"
-
-/* One day, this will be on your libdrm. */
-#define DRM_CLIENT_CAP_UNIVERSAL_PLANES 2
 
 #define MSR_PC8_RES	0x630
 #define MSR_PC9_RES	0x631
@@ -88,7 +78,7 @@ enum plane_type {
 #define WAIT_EXTRA	4
 #define USE_DPMS	8
 
-int drm_fd, msr_fd, pm_status_fd, pc8_status_fd;
+int drm_fd, msr_fd, pc8_status_fd;
 bool has_runtime_pm, has_pc8;
 struct mode_set_data ms_data;
 
@@ -670,8 +660,11 @@ static void setup_pc8(void)
 		return;
 
 	pc8_status_fd = igt_debugfs_open("i915_pc8_status", O_RDONLY);
+	if (pc8_status_fd == -1)
+		pc8_status_fd = igt_debugfs_open("i915_runtime_pm_status",
+						 O_RDONLY);
 	igt_assert_f(pc8_status_fd >= 0,
-		     "Can't open /sys/kernel/debug/dri/0/i915_pc8_status");
+		     "Can't open /sys/kernel/debug/dri/0/i915_runtime_pm_status");
 
 	has_pc8 = true;
 }
@@ -706,7 +699,7 @@ static void setup_non_graphics_runtime_pm(void)
 
 static void setup_environment(void)
 {
-	drm_fd = drm_open_any_master();
+	drm_fd = drm_open_driver_master(DRIVER_INTEL);
 
 	init_mode_set_data(&ms_data);
 
@@ -1832,11 +1825,11 @@ int main(int argc, char *argv[])
 			stay_subtest();
 
 	/* Essential things */
-	igt_subtest("rte")
+	igt_subtest("basic-rte")
 		basic_subtest();
 	igt_subtest("drm-resources-equal")
 		drm_resources_equal_subtest();
-	igt_subtest("pci-d3-state")
+	igt_subtest("basic-pci-d3-state")
 		pci_d3_state_subtest();
 
 	/* Basic modeset */
