@@ -49,6 +49,7 @@
 #include "config.h"
 #endif
 
+#include "igt.h"
 #include <cairo.h>
 #include <errno.h>
 #include <getopt.h>
@@ -64,10 +65,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include "ioctl_wrappers.h"
-#include "drmtest.h"
 #include "testdisplay.h"
-#include "igt_kms.h"
 
 #include <stdlib.h>
 #include <signal.h>
@@ -240,10 +238,7 @@ paint_color_key(struct igt_fb *fb_info)
 
 static void paint_image(cairo_t *cr, const char *file)
 {
-	int img_x, img_y, img_w, img_h, img_w_o, img_h_o;
-	double img_w_scale, img_h_scale;
-
-	cairo_surface_t *image;
+	int img_x, img_y, img_w, img_h;
 
 	img_y = height * (0.10 );
 	img_h = height * 0.08 * 4;
@@ -251,22 +246,7 @@ static void paint_image(cairo_t *cr, const char *file)
 
 	img_x = (width / 2) - (img_w / 2);
 
-	image = cairo_image_surface_create_from_png(file);
-
-	img_w_o = cairo_image_surface_get_width(image);
-	img_h_o = cairo_image_surface_get_height(image);
-
-	cairo_translate(cr, img_x, img_y);
-
-	img_w_scale = (double)img_w / (double)img_w_o;
-	img_h_scale = (double)img_h / (double)img_h_o;
-	cairo_scale(cr, img_w_scale, img_h_scale);
-
-	cairo_set_source_surface(cr, image, 0, 0);
-	cairo_scale(cr, 1, 1);
-
-	cairo_paint(cr);
-	cairo_surface_destroy(image);
+	igt_paint_image(cr, file, img_x, img_y, img_h, img_w);
 }
 
 static void paint_output_info(struct connector *c, struct igt_fb *fb)
@@ -318,7 +298,7 @@ static void paint_output_info(struct connector *c, struct igt_fb *fb)
 	}
 
 	if (qr_code)
-		paint_image(cr, IGT_DATADIR"/pass.png");
+		paint_image(cr, "pass.png");
 
 	igt_assert(!cairo_status(cr));
 
@@ -744,7 +724,7 @@ int main(int argc, char **argv)
 	    specified_mode_num == -1 && !test_stereo_modes)
 		test_all_modes = 1;
 
-	drm_fd = drm_open_any();
+	drm_fd = drm_open_driver(DRIVER_INTEL);
 
 	if (test_stereo_modes &&
 	    drmSetClientCap(drm_fd, DRM_CLIENT_CAP_STEREO_3D, 1) < 0) {
